@@ -23,13 +23,16 @@ import os
 
 import markdown
 import jinja2
+from warnings import warn
 
 from markdown_checklist.extension import ChecklistExtension
 
 from urubu import UrubuWarning, UrubuError
 from urubu import md_extensions
 
-layoutdir = "_layouts"
+from urubu.config import layoutdir, tag_layout 
+
+tag_layout_warning = "Tags defined, but no {} layout found".format(tag_layout)
 
 def skip_yamlfm(f):
     """Return source of a file without yaml frontmatter."""
@@ -47,8 +50,9 @@ class ContentProcessor(object):
 
     def __init__(self, sitedir, project):
         self.sitedir = sitedir
-        self.fileinfo = project.fileinfo
-        self.navinfo = project.navinfo
+        self.filelist = project.filelist
+        self.navlist = project.navlist
+        self.taglist = project.taglist
         self.site = project.site
         tableclass = md_extensions.TableClassExtension() 
         projectref = md_extensions.ProjectReferenceExtension()
@@ -69,7 +73,13 @@ class ContentProcessor(object):
         self.templates = {}
         for layout in project.layouts:
             self.templates[layout] = self.env.get_template(layout + '.html')
-
+        # layout for tags is optional, triggers index file generation per tag
+        try:
+            self.templates[tag_layout] = self.env.get_template(tag_layout + '.html')
+        except jinja2.exceptions.TemplateNotFound:
+            if self.taglist:
+                warn(tag_layout_warning, UrubuWarning)
+            
 
     def process(self):
         """Process the content.
@@ -81,7 +91,7 @@ class ContentProcessor(object):
         self.render() 
         
     def convert(self):
-        for info in self.fileinfo:
+        for info in self.filelist:
             fn = info['fn']
             with open(fn, encoding='utf-8-sig') as inf:
                src = skip_yamlfm(inf)
@@ -99,7 +109,7 @@ class ContentProcessor(object):
                 key = mdkey[:-3]
                 info[key] = self.md.convert(info[mdkey]) 
             self.md.reset()        
-        for info in self.navinfo:
+        for info in self.navlist:
             # markdown support in keys
             mdkeys = [key for key in info if key[-3:] == '.md']
             for mdkey in mdkeys:
@@ -107,16 +117,24 @@ class ContentProcessor(object):
                 info[key] = self.md.convert(info[mdkey]) 
             self.md.reset()        
 
+    def render_file(self, info):
+        layout = info['layout']
+        templ = self.templates[layout]
+        html = templ.render(this=info, site=self.site)
+        fn = info['fn']
+        bfn, ext = os.path.splitext(fn)
+        outfn = os.path.join(self.sitedir, bfn) + self.site['file_ext']
+        with open(outfn, 'w', encoding='utf-8', errors='strict') as outf:
+           outf.write(html)
+
     def render(self):
-        for info in self.fileinfo:
-            layout = info['layout']
-            if layout is None:
+        # content files
+        for info in self.filelist:
+            if info['layout'] is None:
                 continue
-            templ = self.templates[layout]
-            html = templ.render(this=info, site=self.site)
-            fn = info['fn']
-            bfn, ext = os.path.splitext(fn)
-            outfn = os.path.join(self.sitedir, bfn) + self.site['file_ext']
-            with open(outfn, 'w', encoding='utf-8', errors='strict') as outf:
-               outf.write(html)
-        
+            self.render_file(info)
+        # tag index files
+        if tag_layout not in self.templates:
+            return
+        for info in self.taglist:
+            self.render_file(info)
