@@ -30,10 +30,8 @@ import itertools
 from urubu import UrubuWarning, UrubuError
 from urubu import readers, processors 
 
-configfn = '_config.yml'
-siteinfofn = '_site.yml'
-workdir = os.getcwd()
-sitedir = '_build'
+from urubu.config import (configfn, siteinfofn, workdir, sitedir,
+                          tagdir, tagid, tagindexid, tag_layout) 
 
 yamlfm_warning = "No yaml front matter in '{}' - ignored"
 ambig_reflink_error = "Ambiguous reference id '{}'"
@@ -82,14 +80,14 @@ class Project(object):
     def __init__(self):
         self.config = {}
         self.site = {'brand' : '',
-                     'reflinks': {},
+                     'reflinks' : {},
                      'link_ext' : '.html',
                      'file_ext' : '.html'
                     }
-        self.fileinfo = []
-        self.filerefs = {}
-        self.navinfo = [] 
-	self.navrefs = {}
+        self.filelist = []
+        self.navlist = [] 
+        self.taglist = []
+        self.tagmap = {}
         self.filters = {}
         self.validators = {}
         self.layouts = []
@@ -144,7 +142,7 @@ class Project(object):
         self.site['reflinks'][id] = info 
          
     def get_contentinfo(self):
-        """Get info form the markdown content files."""
+        """Get info from the markdown content files."""
         pattern = '*.md'
         for path, dirnames, filenames in os.walk(workdir):
             relpath = get_relpath(path, workdir)
@@ -155,21 +153,31 @@ class Project(object):
                     if meta is None:
                         warn(yamlfm_warning.format(relfn), UrubuWarning)
                         continue
-                    info = self.make_fileinfo(relfn, meta)
-                    self.fileinfo.append(info)
+                    fileinfo = self.make_fileinfo(relfn, meta)
+                    self.filelist.append(fileinfo)
                     # validate after file info has been added so it can be used
-                    self.validate_fileinfo(relfn, info)
-                    self.add_reflink(info['id'], info)
+                    self.validate_fileinfo(relfn, fileinfo)
+                    self.add_reflink(fileinfo['id'], fileinfo)
                     if fn == 'index.md':
-                        info = self.make_navinfo(relpath, meta)
-                        self.navinfo.append(info)
-                        self.validate_navinfo(relfn, info)
-                        self.add_reflink(info['id'], info)
+                        # start from fileinfo of index file
+                        navinfo = self.make_navinfo(relpath, fileinfo)
+                        self.navlist.append(navinfo)
+                        self.validate_navinfo(relfn, navinfo)
+                        self.add_reflink(navinfo['id'], navinfo)
+                        # add nav info to tag map
+                        self.add_info_to_tagmap(navinfo)
+                    else:
+                        # add id for non-index files to tag tags
+                        self.add_info_to_tagmap(fileinfo)
 
     def validate_fileinfo(self, relfn, info):
+        # layout is mandatory
         if 'layout' not in info:
             raise UrubuError(undef_info_error.format('File', relfn, 'layout'))
         layout = info['layout']
+        # modification date, always available
+        t = os.path.getmtime(relfn)
+        info['mdate'] = datetime.date.fromtimestamp(t)
         # first run a validator if it exist
         if layout in self.validators:
             self.validators[layout](info)
@@ -179,15 +187,28 @@ class Project(object):
             return
         if layout not in self.layouts:
             self.layouts.append(layout)
+        # title
         if 'title' not in info:
             raise UrubuError(undef_info_error.format('File', relfn, 'title'))
+        # date
         if 'date' in info:
             if not isinstance(info['date'], datetime.date):
                 raise UrubuError(date_error.format(relfn))
+        # tags
+        if 'tags' in info:
+            # TODO: make sure it's a list of strings
+            if isinstance(info['tags'], str):
+                info['tags'] = [info['tags']]
+            # TODO: normalize tags
 
     def validate_navinfo(self, relfn, info):
         if ('content' not in info) and ('order' not in info):
-            raise UrubuError(undef_content_error.format(relfn))
+            # exception: tag folder doesn't require content atribute
+            # set it to empty list align with normal folders 
+            if info['id'] == tagid:
+                info['content'] = []
+            else:
+                raise UrubuError(undef_content_error.format(relfn))
         require_key('content', info, list, relfn)
 
     def make_fileinfo(self, relfn, meta):
@@ -201,9 +222,11 @@ class Project(object):
         info.update(meta)
         return info
 
-    def make_navinfo(self, relpath, meta):
+    def make_navinfo(self, relpath, fileinfo):
         """Make a navinfo dict."""
-        info = {}
+        # start from fileinfo of index file
+        info = fileinfo.copy() 
+        # overwrite attributes according to navinfo view
         info['fn'] = relpath 
         info['components'] = components = get_components(relpath)
         info['id'] = make_id(components)
@@ -211,11 +234,20 @@ class Project(object):
         info['url'] = info['id']
         if info['url'] != '/':
             info['url'] += '/'
-        info.update(meta)
         return info
 
+    def add_info_to_tagmap(self, info):
+        """Add id to tagmap."""
+        # tags are optional
+        if 'tags' not in info:
+            return
+        for tag in info['tags']:
+            if tag not in self.tagmap:
+                self.tagmap[tag] = []
+            self.tagmap[tag].append(info)
+
     def resolve_reflinks(self):
-        for info in self.navinfo:
+        for info in self.navlist:
             if 'content' in info:
                 self.resolve_content(info)
             else: # order
@@ -280,7 +312,7 @@ class Project(object):
                    raise UrubuError(undef_key_error.format(key, item['fn'])) 
                return True
             return False 
-        allinfo = itertools.chain(self.fileinfo, self.navinfo)
+        allinfo = itertools.chain(self.filelist, self.navlist)
         refcontent = itertools.ifilter(pred, allinfo)
 
         def get_keyval(item):
@@ -290,7 +322,7 @@ class Project(object):
 
 
     def make_breadcrumbs(self):
-        for info in self.fileinfo:
+        for info in self.filelist:
             breadcrumbs = []
             id = '' 
             comps = info['components'] 
@@ -303,7 +335,7 @@ class Project(object):
             info['breadcrumbs'] = breadcrumbs
 
     def make_pager(self):
-        for info in self.navinfo:
+        for info in self.navlist:
             content = info['content']
             if not content:
                 return
@@ -313,6 +345,47 @@ class Project(object):
                 content[i]['prev'] = content[i-1]
             content[-1]['next'] = None    
 
+    def make_taginfo(self, tag, content):
+        """Make a taginfo dict."""
+        info = {} 
+        info['title'] = info['tag'] = tag
+        info['layout'] = tag_layout
+        info['fn'] = os.path.join(tagdir, tag, 'index') 
+        info['components'] = components = (tagdir, tag) 
+        info['id'] = make_id(components)
+        # add trailing slash for tag index url
+        info['url'] = info['id'] + '/'
+        info['content'] = content
+        return info
+
+    def process_tags(self):
+        """ Process tag map and tag content."""
+        def get_date(item):
+            """ Return item's date, or mdate as fallback."""
+            if 'date' in item:
+                return item['date']
+            else:
+                return item['mdate']
+        for tag in self.tagmap.keys():
+            # sort tag content by date
+            content = sorted(self.tagmap[tag], key=get_date, reverse=True)
+            taginfo = self.make_taginfo(tag, content)
+            self.taglist.append(taginfo)
+            self.add_reflink(taginfo['id'], taginfo)
+
+        # sort tags according to content length
+        def get_contentlen(taginfo):
+            return len(taginfo['content'])
+        self.taglist = sorted(self.taglist, key=get_contentlen, reverse=True)
+                
+        # set up tagid info dict if it doesn't exist already
+        if tagid not in self.site['reflinks']:
+            self.site['reflinks'][tagid] = {}
+        self.site['reflinks'][tagid]['content'] = self.taglist 
+        # propagate content to index file if it exists
+        if tagindexid in self.site['reflinks']:
+            self.site['reflinks'][tagindexid]['content'] = self.taglist 
+
     def make_site(self):
         """Make the site."""
         # Keep sitedir alive if it exists, for the server
@@ -329,7 +402,14 @@ class Project(object):
             if os.path.isdir(wp):
                 shutil.copytree(wp, sp, ignore=ignore)
             elif os.path.isfile(wp):
-                shutil.copyfile(wp, sp)            
+                shutil.copyfile(wp, sp) 
+        # make tag index dirs
+        if self.taglist:
+            tagpath = os.path.join(sitedir, tagdir)
+            if not os.path.isdir(tagpath):
+                os.mkdir(tagpath)
+            for taginfo in self.taglist:
+                os.mkdir(os.path.join(tagpath, taginfo['tag']))
         self.process_content()
 
     def process_content(self):
@@ -349,4 +429,5 @@ def build():
     # pprint.pprint(proj.site['reflinks'])
     proj.make_breadcrumbs()
     proj.make_pager()
+    proj.process_tags()
     proj.make_site()
